using System;
using System.Diagnostics;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;


namespace HealthBarProject
{

    public class Camera : ICloneable
    {

        /// <summary>
        ///		Camera orientation.
        /// </summary>
        internal protected Vector3 orientation;
        /// <summary>
        ///		Camera position.
        /// </summary>
        internal protected Vector3 position;
        /// <summary>
        ///		Camera direction.
        /// </summary>
        internal protected Vector3 direction;
        /// <summary>
        ///     y-direction field-of-view (default 90deg).
        /// </summary>
        internal protected float fieldOfView = MathHelper.PiOver2;
        /// <summary>
        ///     Far clip distance - default 10000.
        /// </summary>
        internal protected float farDistance = 10000;
        /// <summary>
        ///     Near clip distance - default 1.
        /// </summary>
        internal protected float nearDistance = 1;
        /// <summary>
        ///     Pre-calced projection matrix.
        /// </summary>
        protected Matrix projectionMatrix;
        /// <summary>
        ///     Pre-calced view matrix.
        /// </summary>
        protected Matrix viewMatrix;
        protected  Game1 game;


        public Game1 Game
        {
            get { return game; }
        }     	

        public float NearDistance
        {
            get { return this.nearDistance; }
            set
            {
                this.nearDistance = value;
            }
        }

        public float FarDistance
        {
            get { return this.farDistance; }
            set
            {
                this.farDistance = value;
            }
        }

        public float FieldOfView
        {
            get { return this.fieldOfView; }
            set
            {
                this.fieldOfView = value;
            }
        }

        public Matrix ViewMatrix
        {
            get { return viewMatrix; }
        }

        public Matrix ProjectionMatrix
        {
            get { return projectionMatrix; }
        }

        public Camera(Game1 game)
        {
            // Init camera location & direction
            this.game = game;

            // Locate at (0,0,0)
            position = Vector3.Backward  * 100;
            position += Vector3.Up * 1000;
            direction = Vector3.Zero;
            orientation = Vector3.Up;

            fieldOfView = MathHelper.PiOver2;
            nearDistance = 1.0f;
            farDistance = 10000.0f;

            viewMatrix = Matrix.Identity;
            projectionMatrix = Matrix.Identity;
            this.Update(null);
        }

        public float LookDistance()
        {
            return Vector3.Distance(this.position, this.direction);
        }

        /// <summary>
        ///     Gets/Sets the camera's orientation.
        /// </summary>
        public Vector3 Orientation
        {
            get { return orientation; }
            set { orientation = value; }
        }

        /// <summary>
        ///     Gets/Sets the camera's position.
        /// </summary>
        public Vector3 Position
        {
            get { return this.position; }
            set { this.position = value; }
        }

        /// <summary>
        ///		Gets/Sets the camera's direction vector.
        /// </summary>
        public Vector3 Direction
        {
            get { return direction; }
            set { direction = value; }
        }


        /// <summary>
        /// Moves the camera's position by the vector offset provided along world axes.
        /// </summary>
        /// <param name="offset"></param>
        public void Move(Vector3 offset)
        {
            position = position + offset;
        }

        /// <summary>
        /// Moves the camera's position by the vector offset provided along it's own axes (relative to orientation).
        /// </summary>
        /// <param name="offset"></param>
        public void MoveRelative(Vector3 offset)
        {
            // Transform the axes of the relative vector by camera's local axes
            Vector3 transform = orientation * offset;

            position = position + transform;
        }

        /// <summary>
        ///		Specifies a target that the camera should look at.
        /// </summary>
        /// <param name="target"></param>
        public void LookAt(Vector3 target)
        {
            this.Direction = target;// (target - derivedPosition);
        }

        /// <summary>
        ///		Pitches the camera up/down counter-clockwise around it's local x axis.
        /// </summary>
        /// <param name="degrees"></param>
        public void Pitch(float degrees)
        {
            Vector3 xAxis = orientation * Vector3.UnitX;
            Rotate(xAxis, degrees);
        }

        /// <summary>
        ///		Rolls the camera counter-clockwise, in degrees, around its local y axis.
        /// </summary>
        /// <param name="degrees"></param>
        public void Yaw(float degrees)
        {
            Vector3 yAxis;
            yAxis = orientation * Vector3.UnitY;

            Rotate(yAxis, degrees);
        }

        /// <summary>
        ///		Rolls the camera counter-clockwise, in degrees, around its local z axis.
        /// </summary>
        /// <param name="degrees"></param>
        public void Roll(float degrees)
        {
            // Rotate around local Z axis
            Vector3 zAxis = orientation * Vector3.UnitZ;
            Rotate(zAxis, degrees);
        }

        /// <summary>
        ///		Rotates the camera about an arbitrary axis.
        /// </summary>
        /// <param name="quat"></param>
        public void Rotate(Quaternion quat)
        {
            // Note the order of the multiplication
            orientation = Helpers.Vec3QuatMultiply(quat, orientation);
        }

        /// <summary>
        ///		Rotates the camera about an arbitrary axis.
        /// </summary>
        /// <param name="axis"></param>
        /// <param name="degrees"></param>
        public void Rotate(Vector3 axis, float degrees)
        {
            Quaternion q = Quaternion.CreateFromAxisAngle(axis, MathHelper.ToRadians(degrees));
            Rotate(q);
        }

        public object Clone()
        {
            Camera c = new Camera(this.game);
            c.direction = this.direction;
            c.farDistance = this.farDistance;
            c.fieldOfView = this.fieldOfView;
            c.nearDistance = this.nearDistance;
            c.orientation = this.orientation;
            c.position = this.position;
            c.projectionMatrix = this.projectionMatrix;
            c.viewMatrix = this.viewMatrix;
            return c;
        }


        public void Update(GameTime gameTime)
        {
            float aspectRatio ;
            int w= game.graphics.GraphicsDevice.Viewport.Width;
            int h= game.graphics.GraphicsDevice.Viewport.Height;
            aspectRatio = w / h;
            //if (w > h) { aspectRatio = w / h; } else { aspectRatio = h / w; }

            viewMatrix = Matrix.CreateLookAt(this.position, this.direction, this.orientation);
            projectionMatrix = Matrix.CreatePerspectiveFieldOfView(this.fieldOfView, aspectRatio, this.nearDistance, this.farDistance);
        }

       
    }
}
