#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace HealthBarProject
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public  class GridRenderer : Microsoft.Xna.Framework.DrawableGameComponent
    {

        public enum GridType
        {
            Points,
            Lines
        }

        internal protected float cellWidth = 16;
        internal protected float cellHeight = 16;
        internal protected float gridWidth = 800;
        internal protected float gridHeight = 600;
        internal protected GridType gridStyle = GridType.Lines;
        private Color color = Color.Blue;
        private bool needsInit = true;
        private Vector3 offset;
        private VertexDeclaration decl;
        private int vertCount = 0;
        private VertexBuffer vb;
        private BasicEffect fx;
        private float pointSize = 4;
        private float yaw = 0;
        private float pitch = 0;
        private float roll = 0;
        private Camera camera;

        public Camera Camera
        {
            get { return camera; }
            set { camera = value; }
        }
	

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
            this.fx.Dispose();
            this.fx = null;
            this.decl.Dispose();
            this.decl = null;
            this.vb.Dispose();
            this.vb = null;
        }

        public float Yaw
        {
            get { return yaw; }
            set { yaw = value; }
        }

        public float Pitch
        {
            get { return pitch; }
            set { pitch = value; }
        }

        public float Roll
        {
            get { return roll; }
            set { roll = value; }
        }

        public float PointSize
        {
            get { return pointSize; }
            set { pointSize = value; }
        }

        public Vector3 Offset
        {
            get { return offset; }
            set { offset = value; }
        }

        public Color GridColor
        {
            get { return color; }
            set { color = value; }
        }

        public GridType GridStyle
        {
            get { return gridStyle; }
            set
            {
                if (this.gridStyle == value) return;
                gridStyle = value;
                needsInit = true;
            }
        }

        public float CellWidth
        {
            get { return cellWidth; }
            set
            {
                cellWidth = value;
                needsInit = true;
            }
        }

        public float CellHeight
        {
            get { return cellHeight; }
            set
            {
                cellHeight = value;
                needsInit = true;
            }
        }

        public float GridWidth
        {
            get { return cellWidth; }
            set
            {
                gridWidth = value;
                needsInit = true;
            }
        }

        public float GridHeight
        {
            get { return cellHeight; }
            set
            {
                gridHeight = value;
                needsInit = true;
            }
        }

        public void SetGridSize(float w, float h)
        {
            this.gridWidth = w;
            this.gridHeight = h;
            this.needsInit = true;
        }

        public void SetCellSize(float w, float h)
        {
            this.cellWidth = w;
            this.cellHeight = h;
            this.needsInit = true;
        }

        public GridRenderer(Game game, float gw, float gh)
            : base(game)
        {
             this.gridWidth = gw;
            this.gridHeight = gh;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            base.Initialize();
            if (decl == null) decl = new VertexDeclaration(this.GraphicsDevice, VertexPositionColor.VertexElements);
            if (fx == null) fx = new BasicEffect(this.GraphicsDevice, null);
            // this.camera = this.game.camera;// (Camera)this.game.cam.Clone();
            // this.gridCamera.ProjectionType = Projection.Orthographic;
            if (needsInit) { InitGrid(); }
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            if (needsInit) { InitGrid(); }

            base.Update(gameTime);
        }

        private void InitGrid()
        {
            int count = 0;
            if (gridStyle == GridType.Points)
            {
                count = (int)((this.gridWidth / this.cellWidth) + 1);
                count *= (int)((this.gridHeight / this.cellHeight) + 1);
                count++;
            }
            else
            {
                count = (int)((this.gridWidth / this.cellWidth) * 2);
                count = count + (int)((this.gridHeight / this.cellHeight) * 2);
                count += 4;
            }
            VertexPositionColor[] verts = new VertexPositionColor[count];
            // dispose vb if prev created
            if (vb != null) { vb.Dispose(); vb = null; }
            // create new vb
            vb = new VertexBuffer(this.GraphicsDevice, verts.Length * VertexPositionColor.SizeInBytes, ResourceUsage.None, ResourceManagementMode.Automatic);

            int index = 0;
            switch (this.gridStyle)
            {
                case GridType.Lines:
                    // generate vert lines
                    for (float idy = 0; idy <= this.gridHeight; idy += this.cellHeight)
                    {
                        verts[index] = new VertexPositionColor(new Vector3(0, idy, 0), this.color);
                        verts[index + 1] = new VertexPositionColor(new Vector3(this.gridWidth, idy, 0), this.color);
                        index += 2;
                    }

                    // generate horiz lines
                    for (float idy = 0; idy <= this.gridWidth; idy += this.cellWidth)
                    {
                        verts[index] = new VertexPositionColor(new Vector3(idy, 0, 0), this.color);
                        verts[index + 1] = new VertexPositionColor(new Vector3(idy, this.gridHeight, 0), this.color);
                        index += 2;
                    }
                    break;

                case GridType.Points:
                    for (float idy = 0; idy <= this.gridHeight; idy += this.cellHeight)
                    {
                        for (float idx = 0; idx <= this.gridWidth; idx += this.cellWidth)
                        {
                            verts[index] = new VertexPositionColor(new Vector3(idx, idy, 0), this.color);
                            index++;
                        }
                    }
                    break;
            }

            // populate vertex buffer
            vb.SetData<VertexPositionColor>(verts);
            vertCount = verts.Length;
            this.needsInit = false;
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // TODO: Add your draw code here
            GraphicsDevice g = this.GraphicsDevice;
            g.VertexDeclaration = decl;
            g.Vertices[0].SetSource(vb, 0, VertexPositionColor.SizeInBytes);


            Matrix baseWorld = Matrix.CreateRotationX(this.yaw);
            baseWorld *= Matrix.CreateRotationX(this.pitch);
            baseWorld *= Matrix.CreateRotationX(this.roll);
            baseWorld *= Matrix.CreateTranslation(-this.offset);
            fx.World = baseWorld;
            fx.VertexColorEnabled = true;
            fx.TextureEnabled = false;

                    fx.View = camera.ViewMatrix;
                    fx.Projection = camera.ProjectionMatrix;

                    fx.Begin(SaveStateMode.SaveState);
                    foreach (EffectPass pass in fx.CurrentTechnique.Passes)
                    {
                        pass.Begin();
                        switch (this.gridStyle)
                        {
                            case GridType.Lines:
                                g.DrawPrimitives(PrimitiveType.LineList, 0, vertCount / 2);
                                break;

                            case GridType.Points:
                                g.RenderState.PointSize = this.pointSize;
                                g.DrawPrimitives(PrimitiveType.PointList, 0, vertCount);
                                break;
                        }
                        pass.End();
                    }
                    fx.End();
           
           
            base.Draw(gameTime);
        }
    }
}


